@@ -12,7 +12,7 @@ from django.utils.translation import ugettext_lazy as _
 
 from pyfb_direction.models import Destination
 from pyfb_endpoint.models import CustomerEndpoint, ProviderEndpoint
-from pyfb_kamailio.models import RtpEngine, Acc
+from pyfb_kamailio.models import RtpEngine, AccCdr
 from pyfb_routing.models import RoutingGroup
 from pyfb_rating.models import CustomerRatecard, ProviderRatecard, CustomerCountryRate, CustomerCountryTypeRate, CustomerDefaultRate, CustomerDestinationRate, CustomerRegionRate, CustomerRegionTypeRate, CustomerPrefixRate
 from pyfb_company.models import Customer, Provider
@@ -111,31 +111,31 @@ class RREvent(UUIDModel):
 class CDR(UUIDModel):
 
     # Fields
-    customer_ip = models.CharField(_(u"customer IP address"), max_length=100, null=True, help_text=_(u"Customer IP address."))
-    aleg_uuid = models.CharField(_(u"a leg call-ID"), max_length=100, null=True)
-    caller_number = models.CharField(_(u"caller ID num"), max_length=100, null=True)
-    callee_number = models.CharField(_(u"Dest. number"), max_length=100, null=True)
-    start_time = models.DateTimeField(_(u"start time"), null=True, db_index=True)
+    customer_ip = models.CharField(_(u"customer IP address"), max_length=100, default="", help_text=_(u"Customer IP address."))
+    aleg_uuid = models.CharField(_(u"a leg call-ID"), max_length=100, default="")
+    caller_number = models.CharField(_(u"caller ID num"), max_length=100, default="")
+    callee_number = models.CharField(_(u"Dest. number"), max_length=100, default="")
+    start_time = models.DateTimeField(_(u"start time"), db_index=True)
     answered_time = models.DateTimeField(_(u"answered time"), null=True)
-    end_time = models.DateTimeField(_(u"hangup time"), null=True)
-    duration = models.DecimalField(_(u"call duration"), max_digits=13, decimal_places=3, null=True, help_text=_(u"effective call duration in s."))
+    end_time = models.DateTimeField(_(u"hangup time"))
+    duration = models.DecimalField(_(u"call duration"), max_digits=13, decimal_places=3, default=0, help_text=_(u"effective call duration in s."))
     billsec = models.IntegerField(_(u"billed duration"), default=0, help_text=_(u"billed call duration in s."))
-    read_codec = models.CharField(_(u"read codec"), max_length=20, null=True)
-    write_codec = models.CharField(_(u"write codec"), max_length=20, null=True)
-    sip_code = models.CharField(_(u"hangup SIP code"), max_length=3, null=True, db_index=True)
-    sip_reason = models.TextField(_(u"hangup SIP reason"), max_length=255, null=True)
-    cost_rate = models.DecimalField(_(u'buy rate'), max_digits=11, decimal_places=5, default="0", null=True)
-    total_sell = models.DecimalField(_(u'total sell'), max_digits=11, decimal_places=5, default="0", null=True)
-    total_cost = models.DecimalField(_(u'total cost'), max_digits=11, decimal_places=5, default="0", null=True)
-    rate = models.DecimalField(_(u'sell rate'), max_digits=11, decimal_places=5, null=True)
-    sip_charge_info = models.CharField( _(u'charge info'), null=True, max_length=100, help_text=_(u"Contents of the P-Charge-Info header for billing purpose."))
-    sip_user_agent = models.CharField(_(u'sip user agent'), null=True, max_length=100)
-    sip_rtp_rxstat = models.CharField(_(u'sip rtp rx stat'), null=True, max_length=30)
-    sip_rtp_txstat = models.CharField(_(u'sip rtp tx stat'), null=True, max_length=30)
+    read_codec = models.CharField(_(u"read codec"), max_length=20, default="")
+    write_codec = models.CharField(_(u"write codec"), max_length=20, default="")
+    sip_code = models.CharField(_(u"hangup SIP code"), max_length=3, default="", db_index=True)
+    sip_reason = models.TextField(_(u"hangup SIP reason"), max_length=255, default="")
+    cost_rate = models.DecimalField(_(u'buy rate'), max_digits=11, decimal_places=5, default=0)
+    total_sell = models.DecimalField(_(u'total sell'), max_digits=11, decimal_places=5, default=0)
+    total_cost = models.DecimalField(_(u'total cost'), max_digits=11, decimal_places=5, default=0)
+    rate = models.DecimalField(_(u'sell rate'), max_digits=11, decimal_places=5, default=0)
+    sip_charge_info = models.CharField( _(u'charge info'), default="", max_length=100, help_text=_(u"Contents of the P-Charge-Info header for billing purpose."))
+    sip_user_agent = models.CharField(_(u'sip user agent'), default="", max_length=100)
+    sip_rtp_rxstat = models.CharField(_(u'sip rtp rx stat'), default="", max_length=30)
+    sip_rtp_txstat = models.CharField(_(u'sip rtp tx stat'), default="", max_length=30)
     kamailio_server = models.IntegerField(_(u"SIP server"), default=1)
-    hangup_disposition = models.CharField(_(u"hangup disposition"), null=True, default="", max_length=100)
-    direction = models.CharField(_(u"Type of call"), max_length=10, null=True, blank=True, help_text=_(u"""Type of calls."""))
-    call_class = models.CharField(_(u"Class of call"), max_length=10, null=True, blank=True, help_text=_(u"""Class of calls."""))
+    hangup_disposition = models.CharField(_(u"hangup disposition"), default="", max_length=100)
+    direction = models.CharField(_(u"Type of call"), max_length=10, default="", help_text=_(u"""Type of calls."""))
+    call_class = models.CharField(_(u"Class of call"), max_length=10, default="", help_text=_(u"""Class of calls."""))
     rated = models.DateTimeField(_(u"time cdr was rated"), null=True)
 
     # Relationship Fields
@@ -144,6 +144,7 @@ class CDR(UUIDModel):
         on_delete=models.CASCADE,
         verbose_name=_(u"customer"),
         null=True,
+        blank=True,
         related_name="customer_related"
     )
     provider = models.ForeignKey(
@@ -151,6 +152,7 @@ class CDR(UUIDModel):
         on_delete=models.CASCADE,
         verbose_name=_(u"provider"),
         null=True,
+        blank=True,
         related_name="carrier_related"
     )
     caller_destination = models.ForeignKey(
@@ -179,7 +181,7 @@ class CDR(UUIDModel):
         db_index=False
     )
     cdr_acc = models.ForeignKey(
-        Acc,
+        AccCdr,
         on_delete=models.SET_NULL,
         related_name="accs",
         verbose_name=_(u"Gross CDR"),
@@ -225,6 +227,20 @@ class CDR(UUIDModel):
             color,
             self.sip_code
         )
+    sip_code_colored.short_description = "SIP code"
+
+    def margin(self):
+        margin_value = self.total_sell - self.total_cost
+        if margin_value > 0:
+            color = "black"
+        else:
+            color = "red"
+        return format_html(
+            '<span style=color:{}>{}</span>',
+            color,
+            margin_value
+        )
+    margin.short_description = "Margin"
 
 
 class DimDate(models.Model):
